Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Minicaml.InterfaceSourcetype location = | Location of Lexing.position * Lexing.positiondelimited location
*)| Nowhereno location
*)Exception Error (loc, err, msg) indicates an error of type err with error message msg, occurring at location loc.
error ~loc ~kind raises an error of the given kind. The kfprintf magic allows one to write msg using a format string.
val print_message :
?color:T.color ->
?loc:location ->
string ->
('a, Format.formatter, unit, unit) format4 ->
'aPrint a message at a given location loc of message type msg_type.
A fatal error reported by the toplevel.
A syntax error reported by the toplevel
Parser wrapper that catches syntax-related errors and converts them to errors.