Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Minicaml.TypesSourceA value identifier
type expr = | Unit| Integer of int| Boolean of bool| String of string| Symbol of ide| List of expr list| Cons of expr * expr| Dict of (expr * expr) list| Plus of expr * expr| Sub of expr * expr| Mult of expr * expr| Eq of expr * expr| Gt of expr * expr| Lt of expr * expr| Ge of expr * expr| Le of expr * expr| And of expr * expr| Or of expr * expr| Not of expr| IfThenElse of expr * expr * expr| Let of (ide * expr) list * expr| Letlazy of (ide * expr) list * expr| Letrec of ide * expr * expr| Letreclazy of ide * expr * expr| Lambda of ide list * expr| Apply of expr * expr list| Sequence of expr list| Pipe of expr * exprThe type representing Abstract Syntax Tree expressions
A type useful for evaluating files, stating if a command is an expression or simply a "global" declaration (appended to environment)
val pp_command :
Ppx_deriving_runtime.Format.formatter ->
command ->
Ppx_deriving_runtime.unitA purely functional environment type, parametrized
val pp_env_t :
'a. (Ppx_deriving_runtime.Format.formatter ->
'a ->
Ppx_deriving_runtime.unit) ->
Ppx_deriving_runtime.Format.formatter ->
'a env_t ->
Ppx_deriving_runtime.unitval show_env_t :
'a. (Ppx_deriving_runtime.Format.formatter ->
'a ->
Ppx_deriving_runtime.unit) ->
'a env_t ->
Ppx_deriving_runtime.stringval equal_env_t :
'a. ('a -> 'a -> Ppx_deriving_runtime.bool) ->
'a env_t ->
'a env_t ->
Ppx_deriving_runtime.boolval compare_env_t :
'a. ('a -> 'a -> Ppx_deriving_runtime.int) ->
'a env_t ->
'a env_t ->
Ppx_deriving_runtime.inttype evt = | EvtUnit| EvtInt of int| EvtBool of bool| EvtString of string| EvtList of evt list| EvtDict of (evt * evt) list| Closure of ide list * expr * type_wrapper env_tRecClosure keeps the function name in the constructor for recursion
*)| RecClosure of ide * ide list * expr * type_wrapper env_tAbstraction that permits treating primitives as closures
*)| PrimitiveAbstraction of ide * int * type_wrapper env_tA type that represents an evaluated (reduced) value
val pp_type_wrapper :
Ppx_deriving_runtime.Format.formatter ->
type_wrapper ->
Ppx_deriving_runtime.unitAn environment of already evaluated values
A recursive type representing a stacktrace frame
val pp_stackframe :
Ppx_deriving_runtime.Format.formatter ->
stackframe ->
Ppx_deriving_runtime.unitPush an AST expression into a stack
Pop an AST expression from a stack